/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.5   | 06/04/2014 | Initial release version, still BETA.       |
    |---------|------------|--------------------------------------------|

*/

package Output;

import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.file.FileSinkImages;
import org.graphstream.stream.file.FileSinkImages.LayoutPolicy;
import org.graphstream.stream.file.FileSinkImages.OutputType;
import org.graphstream.stream.file.FileSinkImages.RendererType;
import org.graphstream.stream.file.FileSinkImages.Resolutions;

import Data.CMx2_Campaign;
import Data.CMx2_CampaignDecisionBlock;

public class CampaignTreeWriter {
	
	private int total_number_of_Nodes = 0;
	
	private int numberOfLayersInGraph = 0;
	
	private Node[] Campaign_Node_Array;
	
	private Graph campaign_Graph;
	
	private FileSinkImages outputPicture; 
	
	private String returnNodeName(int number, boolean isCampaignEnd){
		
		String node_number_prefix;
		
		if (number <= 9){
			
			node_number_prefix = "#0";
			
		} else {
							
			node_number_prefix = "#";
			
		}
		
		if (isCampaignEnd) {
			
			return "CE " + node_number_prefix + number;
			
		} else {
			
			return "CB " + node_number_prefix + number;
			
		}
		
	}
	
	private void initialiseGraph(){
		
		campaign_Graph = new SingleGraph("CMx2_Campaign");
		
		campaign_Graph.addAttribute("ui.quality");
		campaign_Graph.addAttribute("ui.antialias");
    	//graph.addAttribute("layout.quality", 4);
    	//graph.addAttribute("layout.stabilization-limit", 1 );
		campaign_Graph.addAttribute("ui.stylesheet",
				"graph {padding: 100px; }" +
    			"node { fill-color: black; text-size: 14; } " +
    			"node.start { fill-color: blue; }" +
    			"node.CE { fill-color: red; }" +
    			"edge { text-size: 14; }" +
    			"edge.win { fill-color: green; }" +
    			"edge.defeat { fill-color: red; }" );
		
	}
	
	private void initialiseOutputPicture(){
		
		//FileSinkImages.CustomResolution res = new FileSinkImages.CustomResolution(1400,400);
		
		outputPicture = new FileSinkImages(OutputType.PNG, Resolutions.HD1080);
		
		outputPicture.setRenderer(RendererType.SCALA);
		
		outputPicture.setLayoutPolicy(LayoutPolicy.NO_LAYOUT);
		
	}
	
	private Set<Node> assignLevelandreturnNodes(Set<Edge> edgeSet, int currentLevel){
		
		Set<Node> returnNodeSet = new HashSet<Node>();
		
		Iterator<Edge> edgeIterator = edgeSet.iterator();
		
		while (edgeIterator.hasNext()){
			
			Edge nextEdge = edgeIterator.next();
			
			Node assignNode = nextEdge.getTargetNode();
			
			assignNode.addAttribute("y", currentLevel);
									
			returnNodeSet.add(assignNode);
			
		}
		
		return returnNodeSet;
		
	}
	
	private Set<Edge> returnLeavingEdgeSet(Set<Node> nodeSet){
		
		Set<Edge> returnEdgeSet = new HashSet<Edge>();
		
		Iterator<Node> nodeIterator = nodeSet.iterator();
		
		while (nodeIterator.hasNext()){
			
			Node nextNode = nodeIterator.next();
			
			returnEdgeSet.addAll(nextNode.getLeavingEdgeSet());
			
		}
		
		return returnEdgeSet;
		
	}
	
	private void setXinLayer(Set<Node> nodeSet, boolean displaced){
		
		Iterator<Node> nodeIterator = nodeSet.iterator();
		
		int current_x = 0;
		
		if (displaced){
			
			current_x = 2;
			
		}		
		
		while (nodeIterator.hasNext()){
			
			Node assignNode = nodeIterator.next();
		
			assignNode.addAttribute("x", current_x);
			
			if (current_x%4==0){
				
				assignNode.addAttribute("ui.style", "text-alignment: above;");
				
			} else {
				
				
				assignNode.addAttribute("ui.style", "text-alignment: under;");
			}
			
			current_x = current_x + 2;
			
		}		
		
	}
	
	private void distribute(){
		
		Node root = Campaign_Node_Array[0];
		
		root.addAttribute("xy", 0, 0);
		
		root.addAttribute("ui.style", "text-alignment: above;");
		
		Set<Node> currentNodeSet = new HashSet<Node>();
		
		currentNodeSet.add(root);
		
		int currentLevel = -1;
		
		numberOfLayersInGraph = 0;
		
		while(!currentNodeSet.isEmpty()){
			
			currentNodeSet = assignLevelandreturnNodes(returnLeavingEdgeSet(currentNodeSet), currentLevel);

			numberOfLayersInGraph++;
			
			currentLevel--;
			
		}
		
		boolean displaced = false;
		
		for(int currentLayer = 1; currentLayer <= numberOfLayersInGraph; currentLayer++){
			
			currentNodeSet = new HashSet<Node>();
		
			for(int currentNode = 0; currentNode < total_number_of_Nodes; currentNode++){
			
				if(Campaign_Node_Array[currentNode] != null){
					
					if(Campaign_Node_Array[currentNode].getNumber("y") == -currentLayer){
						
						currentNodeSet.add(Campaign_Node_Array[currentNode]);
						
					}
					
				}
				
			}
			
			setXinLayer(currentNodeSet, displaced);
			
			displaced = !displaced;
			
		}
		
	}
	
	public void writeCampaignTreeOutput(CMx2_Campaign campaign) throws IOException{
		
		initialiseOutputPicture();
		
		initialiseGraph();
		
		total_number_of_Nodes = campaign.getNumberOfScenarios() + campaign.getNumberOfCampaignEnds();
		
		Campaign_Node_Array = new Node[total_number_of_Nodes];
		Edge[] Campaign_Edge_Array = new Edge[total_number_of_Nodes * 2];
		
		int next_free_node = 0;
		int next_free_edge = 0;
		
		int current_campaign_end = 1;
		
		for(int currentScenarioNumber = 0; currentScenarioNumber < campaign.getNumberOfScenarios(); currentScenarioNumber++){
			
			CMx2_CampaignDecisionBlock currentDecisionBlock = campaign.getCMBNCampaignDecisionBlock(currentScenarioNumber);
			
			if (currentDecisionBlock.getcontainsScenario()){
				
				String current_battle_string, next_end_string;
				
				current_battle_string = returnNodeName(currentScenarioNumber, false);
				
				Campaign_Node_Array[next_free_node] = campaign_Graph.addNode(current_battle_string);
				
				String scenario_Name = campaign.getCampaignScenario(currentScenarioNumber).getTitle();
				
				Campaign_Node_Array[next_free_node].addAttribute("ui.label", "[" + current_battle_string + "] " + scenario_Name);
				
				Campaign_Node_Array[next_free_node].addAttribute("layout.frozen");
				
				if (currentScenarioNumber == 0){
					
					Campaign_Node_Array[next_free_node].addAttribute("ui.class", "start");
					
				}
		
				next_free_node++;
				
				if (!((currentDecisionBlock.getnextBattleWin() != 65535) && (currentDecisionBlock.getnextBattleLose() != 65535))){
					
					next_end_string = returnNodeName(current_campaign_end, true);
					
					Campaign_Node_Array[next_free_node] = campaign_Graph.addNode(next_end_string);
					
					current_campaign_end++;
					
					String campaign_End_Victory_Level_Suffix = "";
					
					if(currentDecisionBlock.getmaximumAllowedCampaignVictory() != "Don't care / determined from overall score"){
						
						campaign_End_Victory_Level_Suffix = " (MA: " + currentDecisionBlock.getShortMaximumAllowedCampaignVictory() + " )";
						
					} else if (currentDecisionBlock.getminimumRequiredCampaignVictory() != "Don't care / determined from overall score"){
						
						campaign_End_Victory_Level_Suffix = " (MR: " + currentDecisionBlock.getShortMinimumRequiredCampaignVictory() + " )";
						
					}
					
					Campaign_Node_Array[next_free_node].addAttribute("ui.label", "CE" + campaign_End_Victory_Level_Suffix);
					
					Campaign_Node_Array[next_free_node].addAttribute("ui.class", "CE");
					
					Campaign_Node_Array[next_free_node].addAttribute("layout.frozen");
					
					next_free_node++;
					
					if ((currentDecisionBlock.getnextBattleWin() == 65535) && (currentDecisionBlock.getnextBattleLose() == 65535)){
						
						Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_end_string, current_battle_string, next_end_string, true);
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.label", currentDecisionBlock.getShortWinThreshold());
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "win");
						
						next_free_edge++;
						
					} else if ((currentDecisionBlock.getnextBattleWin() == 65535) && (currentDecisionBlock.getnextBattleLose() != 65535)) {
						
						Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_end_string, current_battle_string, next_end_string, true);
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.label", currentDecisionBlock.getShortWinThreshold());
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "win");
						
						next_free_edge++;
						
					} else if ((currentDecisionBlock.getnextBattleWin() != 65535) && (currentDecisionBlock.getnextBattleLose() == 65535)) {
						
						Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_end_string, current_battle_string, next_end_string, true);
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "defeat");
						
						next_free_edge++;
						
					}
					
				}
				
			}
			
		}
		
		for(int currentScenarioNumber = 0; currentScenarioNumber < campaign.getNumberOfScenarios(); currentScenarioNumber++){
			
			CMx2_CampaignDecisionBlock currentDecisionBlock = campaign.getCMBNCampaignDecisionBlock(currentScenarioNumber);
			
			if (currentDecisionBlock.getcontainsScenario()){
				
				String current_battle_string, next_win_battle_string, next_lose_battle_string;
				
				current_battle_string = returnNodeName(currentScenarioNumber, false);
				
				if ((currentDecisionBlock.getnextBattleWin() == 65535) && (currentDecisionBlock.getnextBattleLose() != 65535)){
					
					next_lose_battle_string = returnNodeName(currentDecisionBlock.getnextBattleLose(), false);
					
					Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_lose_battle_string, current_battle_string, next_lose_battle_string, true);
					Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "defeat");
					
					next_free_edge++;
					
				} else if ((currentDecisionBlock.getnextBattleWin() != 65535) && (currentDecisionBlock.getnextBattleLose() == 65535)){
					
					next_win_battle_string = returnNodeName(currentDecisionBlock.getnextBattleWin(), false);
					
					Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_win_battle_string, current_battle_string, next_win_battle_string, true);
					Campaign_Edge_Array[next_free_edge].addAttribute("ui.label", currentDecisionBlock.getShortWinThreshold());
					Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "win");
					
					next_free_edge++;
					
				} else if ((currentDecisionBlock.getnextBattleWin() != 65535) && (currentDecisionBlock.getnextBattleLose() != 65535)){
					
					if (currentDecisionBlock.getnextBattleWin() == currentDecisionBlock.getnextBattleLose()){
						
						next_win_battle_string = returnNodeName(currentDecisionBlock.getnextBattleWin(), false);
						
						Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_win_battle_string, current_battle_string, next_win_battle_string, true);
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.label", currentDecisionBlock.getShortWinThreshold());
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "win");
						
						next_free_edge++;
						
					} else {
						
						next_win_battle_string = returnNodeName(currentDecisionBlock.getnextBattleWin(), false);
						next_lose_battle_string = returnNodeName(currentDecisionBlock.getnextBattleLose(), false);
						
						Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_win_battle_string, current_battle_string, next_win_battle_string, true);
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.label", currentDecisionBlock.getShortWinThreshold());
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "win");
						
						next_free_edge++;
						
						Campaign_Edge_Array[next_free_edge] = campaign_Graph.addEdge(current_battle_string + " " + next_lose_battle_string, current_battle_string, next_lose_battle_string, true);
						Campaign_Edge_Array[next_free_edge].addAttribute("ui.class", "defeat");
						
						next_free_edge++;
						
					}
					
				}
				
			}
			
		}
		
		distribute();
		
		String campaign_filename = campaign.getTitle() + "_Tree.png";
		
		outputPicture.writeAll(campaign_Graph, "campaignpics/" + campaign_filename);
		
		campaign.setCampaignTreeFilename("campaignpics/" + campaign_filename);
		
	}

}
