/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.2   | 17/11/2013 | Added family information / read-out.       |
    |---------|------------|--------------------------------------------|
    |   1.4   | 22/02/2014 | Added further processing for version /     |
    |         |            | module specific offsets and analysis of    |
    |         |            | required modules.                          |
    |---------|------------|--------------------------------------------|
    |  1.5#1  | 23/02/2014 | Changed naming of campaign scenario files  |
    |         |            | to include the internalBattleName.         |
    |---------|------------|--------------------------------------------|
    |  1.5#2  | 05/04/2014 | Added moding necessary for CMRT.           |
    |---------|------------|--------------------------------------------|
    |   1.6   | 27/09/2014 | Added usage of ParsingUtilities provided   |
    |         |            | methods instead of private internal ones.  |
    |---------|------------|--------------------------------------------|
    |   1.7   | 02/11/2014 | Added handling of third module required.   |
    |---------|------------|--------------------------------------------|
    |   1.8   | 23/12/2014 | Added handling of optional modules req'd.  |
    |---------|------------|--------------------------------------------|
    |   1.9   | 27/02/2016 | Added handling of fourth module req'd.     |
    |---------|------------|--------------------------------------------|
*/

package Parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import Data.CMx2_Campaign;
import Data.CMx2_CampaignDecisionBlock;
import Data.CMx2_Scenario;
import Output.CampaignTreeWriter;
import Output.OutputWriter;

public class CampaignReader {

	private final int gameFamilyPosition = 16;
	
	private final int engineVersionPosition = 18;
	
	private final int descSizePosition = 35;
	
    private final int firstModuleRequired_pre_120_Position = 21;
	
	private final int numberOfModulesRequiredPosition = 25;
	
	private final int firstModuleRequiredPosition = 27;
	
	private final int secondModuleRequiredPosition = 28;
	
	private final int thirdModuleRequiredPosition = 29;
	
	private final int fourthModuleRequiredPosition = 30;
	
	private int titleStringSizePosition;	
	private int titleStringPosition;
	
	private int campaignDescSizePosition;	
	private int campaignDescPosition;
	
	private int imageSizePosition;	
	private int imagePosition;
	
	private int modTagBlockPosition;
	
	private int coreUnitBlockSizePosition;
	
	private int numberOfScenariosinCampaignPosition;
	
	private int decisionBlockPosition;
	
	private int internalBattleNamePosition;
	
	private int winThresholdPosition;
	
	private int nextBattleWinPosition;
	
	private int nextBattleLosePosition;
	
	private int minimumRequiredCampaignVictoryPosition;
	
	private int maximumAllowedCampaignVictoryPosition;
	
	private int blueRefitPosition;
	private int blueRepairVehiclePosition;
	private int blueResupplyPosition;
	private int blueRestPosition;
	
	private int redRefitPosition;
	private int redRepairVehiclePosition;
	private int redResupplyPosition;
	private int redRestPosition;
	
	private int scenarioSizePosition;
	private int scenarioPosition;
	
	private static int imageToModTagOffset = 3;
	
	private static int imageToCoreUnitOffset = 21;
	
	private FileInputStream inFile = null;
	
	private FileChannel inChannel = null;
	
	private ByteBuffer oneByteBuffer;
	private ByteBuffer twoByteBuffer;
	private ByteBuffer fourByteBuffer;
	
	private String scenarioDirName;
	
	private ParameterAnalyser parameterAnalyser;
	
	private File campaignPicDir;
	
	private File campaignScenariosDir;
	
	private Set<CMx2_Campaign> campaignList = new HashSet<CMx2_Campaign>();
	
	private Set<CMx2_Scenario> scenarioList = new HashSet<CMx2_Scenario>();
	
	private ParsingUtilities parsingTools;
	
	private void init_File_Channel_Buffers_ParameterAnalyser(String fileName) throws IOException {
		
		inFile = null;
		inChannel = null;
		
		File aFile = new File(fileName);
		inFile = new FileInputStream(aFile);
		
	    inChannel = inFile.getChannel();
	    
	    oneByteBuffer = ByteBuffer.allocate(1);
	    
	    twoByteBuffer = ByteBuffer.allocate(2);
	    
	    fourByteBuffer = ByteBuffer.allocate(4);
	    
	    parameterAnalyser = new ParameterAnalyser();
	    
	    campaignPicDir = new File("campaignpics");
		
		
		if (!campaignPicDir.exists()) {
			
			campaignPicDir.mkdir();
			
		}
		
	}

	private void readSingleCampaign(String currentCampaignFile) throws IOException {
		
		try{

			init_File_Channel_Buffers_ParameterAnalyser(currentCampaignFile);
		
			CMx2_Campaign campaign = new CMx2_Campaign();
	
			byte singleParameter = 0;
		
			int version = 0;
			int family = 0;
			
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, gameFamilyPosition);
			family = parsingTools.getShortfromByte(singleParameter);
			
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, engineVersionPosition);
			version = parsingTools.getShortfromByte(singleParameter);
			
			campaign.setCreatedwithVersion(parameterAnalyser.analyse_and_return_Version(family,version));
			
			int optionalModulesRequiredInt = 0;
	    
			//System.out.println(campaign.getCreatedwithVersion());
		
			int offset = 8; // +8 for normal offset
			
			if (version >= 120) {
				
				offset = offset + 4;
				
				byte[] numberOfModulesRequired = new byte[2];
				byte[] optionalModulesRequired = new byte[2];
				
				numberOfModulesRequired = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, numberOfModulesRequiredPosition);
				
				int numberOfModulesRequiredInt = parsingTools.getShortfromByteArray(numberOfModulesRequired,0);
				
				campaign.setNumberOfModulesRequired(numberOfModulesRequiredInt);
				
				switch(numberOfModulesRequiredInt){
				
					case 0:
						
						optionalModulesRequired = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, numberOfModulesRequiredPosition + 2);
						
						optionalModulesRequiredInt = parsingTools.getShortfromByteArray(optionalModulesRequired,0);
						
						if(optionalModulesRequiredInt == 1){
							
							int firstOptionalModuleRequired = 0;
							int secondOptionalModuleRequired = 0;
							
							firstOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, numberOfModulesRequiredPosition + 4));
							secondOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, numberOfModulesRequiredPosition + 5));
							
							campaign.setOptionalModulesRequired(parameterAnalyser.analyse_and_return_Module_Required(firstOptionalModuleRequired, family) + " or " + parameterAnalyser.analyse_and_return_Module_Required(secondOptionalModuleRequired, family));
							
							offset = offset + 2;
							
						}
						
						break;
				
				    case 1:
				    	
				    	int firstModuleRequired = 0;
				    	firstModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	campaign.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequired, family));
				    	
				    	offset = offset + 1;
				    	
				    	optionalModulesRequired = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, firstModuleRequiredPosition + 1);
						
						optionalModulesRequiredInt = parsingTools.getShortfromByteArray(optionalModulesRequired,0);
						
						if(optionalModulesRequiredInt == 1){
							
							int firstOptionalModuleRequired = 0;
							int secondOptionalModuleRequired = 0;
							
							firstOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition + 3));
							secondOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition + 4));
							
							campaign.setOptionalModulesRequired(parameterAnalyser.analyse_and_return_Module_Required(firstOptionalModuleRequired, family) + " or " + parameterAnalyser.analyse_and_return_Module_Required(secondOptionalModuleRequired, family));
							
							offset = offset + 2;
						}
				    	
				    	break;
				    
				    case 2:
				    	
				    	int firstModuleRequiredOfTwo = 0;
				    	firstModuleRequiredOfTwo = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	campaign.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequiredOfTwo, family));
				    	
				    	int secondModuleRequired = 0;
				    	secondModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, secondModuleRequiredPosition));
				    	campaign.setSecondModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(secondModuleRequired, family));
				    	
				    	offset = offset + 2;
				    	
				    	break;
				    	
				    case 3:
				    	
				    	int firstModuleRequiredOfThree = 0;
				    	firstModuleRequiredOfThree = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	campaign.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequiredOfThree, family));
				    	
				    	int secondModuleRequiredOfThree = 0;
				    	secondModuleRequiredOfThree = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, secondModuleRequiredPosition));
				    	campaign.setSecondModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(secondModuleRequiredOfThree, family));
				    	
				    	int thirdModuleRequired = 0;
				    	thirdModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, thirdModuleRequiredPosition));
				    	campaign.setThirdModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(thirdModuleRequired, family));
				    	
				    	offset = offset + 3;
				    	
				    	break;
				    	
				    case 4:
				    	
				    	int firstModuleRequiredOfFour = 0;
				    	firstModuleRequiredOfFour = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	campaign.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequiredOfFour, family));
				    	
				    	int secondModuleRequiredOfFour = 0;
				    	secondModuleRequiredOfFour = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, secondModuleRequiredPosition));
				    	campaign.setSecondModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(secondModuleRequiredOfFour, family));
				    	
				    	int thirdModuleRequiredOfFour = 0;
				    	thirdModuleRequiredOfFour = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, thirdModuleRequiredPosition));
				    	campaign.setThirdModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(thirdModuleRequiredOfFour, family));
				    	
				    	int fourthModuleRequired = 0;
				    	fourthModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, fourthModuleRequiredPosition));
				    	campaign.setFourthModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(fourthModuleRequired, family));
				    	
				    	offset = offset + 4;
				    	
				    	break;
				    	
				    default: break;
				
				}
				
			} else {
				
				int firstModuleRequired_pre_120 = 0;
				
				firstModuleRequired_pre_120 = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequired_pre_120_Position));
				
				if (firstModuleRequired_pre_120 == 1){
					
					campaign.setNumberOfModulesRequired(1);
					campaign.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequired_pre_120, family));					
					
				}
				
			}
  
			titleStringSizePosition = descSizePosition + offset;
			
			byte[] titleStringSize = new byte[2];
			
			titleStringSize = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, titleStringSizePosition);
	    
			int titleStringSizeInt = parsingTools.getShortfromByteArray(titleStringSize,0);
	    
			if (titleStringSizeInt > 0) {
	    	
				byte[] titleStringByteArray = new byte[titleStringSizeInt];
	    	
				titleStringPosition = titleStringSizePosition + 2;
	    	
				campaignDescSizePosition = titleStringPosition + titleStringSizeInt;
	    	
				titleStringByteArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, titleStringPosition, titleStringSizeInt);
	    	
				campaign.setTitle(parameterAnalyser.convert_ByteArray_to_String(titleStringByteArray));
	    	
				//System.out.println(campaign.getTitle());
	    	
			}else {
	    	
				campaignDescSizePosition = titleStringSizePosition + 2;
	    	
			}
	    
			//System.out.println("-------------------------------------------------");
	    
			//System.out.println(currentCampaignFile);
	    
			int startPos = 0;
			int endPos = currentCampaignFile.lastIndexOf(".cam");
    	
			campaign.setfileName(currentCampaignFile.substring(startPos, endPos));
	    
			scenarioDirName = campaign.getfileName() + " scenarios";
	    
			campaignScenariosDir = new File(scenarioDirName);
		
			if (!campaignScenariosDir.exists()) {
			
				campaignScenariosDir.mkdir();
			
			}
	    
			byte[] campaignDescStringSize = new byte[2];
	    
			campaignDescStringSize = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, campaignDescSizePosition);
	    
			int campaignDescStringSizeInt = parsingTools.getShortfromByteArray(campaignDescStringSize,0);
	    
			if (campaignDescStringSizeInt > 0) {
	    	
				byte[] campaignDescStringByteArray = new byte[campaignDescStringSizeInt];
	    	
				campaignDescPosition = campaignDescSizePosition + 2;
	    	
				imageSizePosition = campaignDescPosition + campaignDescStringSizeInt;
	    	
				campaignDescStringByteArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, campaignDescPosition, campaignDescStringSizeInt);
	    	
				campaign.setDescription(parameterAnalyser.convert_ByteArray_to_String(campaignDescStringByteArray));
	    	
				//System.out.println(campaign.getDescription());
	    	
			}else {
	    	
				imageSizePosition = campaignDescSizePosition + 2;
	    	
			}
	    
			//System.out.println("-------------------------------------------------");
	    
			byte[] imageSize = new byte[4];
	    
			imageSize = parsingTools.getfourByteArrayfromFileChannel(fourByteBuffer, inChannel, imageSizePosition);
	    
			int imageSizeInt = parsingTools.getIntfromByteArray(imageSize,0);
	    
			if (imageSizeInt > 0) {
	    	
				byte[] bitmapImageArray = new byte[imageSizeInt];
	    	
				imagePosition = imageSizePosition + 4;
	    	
				coreUnitBlockSizePosition = imagePosition + imageSizeInt + imageToCoreUnitOffset;
	    	
				bitmapImageArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, imagePosition, imageSizeInt);
	    	
				parsingTools.writebyteArraytoFile(bitmapImageArray, campaign.getfileName(), campaignPicDir, arrayFileType.BMP);
	    	
			} else {
	    	
				coreUnitBlockSizePosition = imageSizePosition + 4 + imageToCoreUnitOffset;
	    	
			}
			
			// mod tag specific 
			int modTagTotalOffset = 0;
	    
			if (version >= 130){
				
				modTagBlockPosition = imagePosition + imageSizeInt + imageToModTagOffset;
				
				modTagTotalOffset = 5;
				
				byte[] numberOfModTags = new byte[4];
				
				numberOfModTags = parsingTools.getfourByteArrayfromFileChannel(fourByteBuffer, inChannel, modTagBlockPosition);
				
				int numberOfModTagsInt = parsingTools.getIntfromByteArray(numberOfModTags,0);
				
				if (numberOfModTagsInt > 0){
					
					String[] modTagsArray = new String[numberOfModTagsInt];
					
					int currentModTagSizePosition = modTagBlockPosition + 4;
					
					for(int currentModTagIndex = 0; currentModTagIndex < numberOfModTagsInt; currentModTagIndex++){
						
						byte[] currentModTagSize = new byte[2];
						
						currentModTagSize = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, currentModTagSizePosition);
						
						int currentModTagSizeInt = parsingTools.getShortfromByteArray(currentModTagSize,0);
						
						if (currentModTagSizeInt>0){
							
							byte[] currentModTagByteArray = new byte[currentModTagSizeInt];
							
							currentModTagByteArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, currentModTagSizePosition + 2, currentModTagSizeInt);
							
							modTagsArray[currentModTagIndex] = parameterAnalyser.convert_ByteArray_to_String(currentModTagByteArray);
							
							currentModTagSizePosition = currentModTagSizePosition + 2 + currentModTagSizeInt;
							
							modTagTotalOffset = modTagTotalOffset + 2 + currentModTagSizeInt;
							
						} else {
							
							currentModTagSizePosition = currentModTagSizePosition + 2;
							
							modTagTotalOffset = modTagTotalOffset + 2;
							
						}
						
					}
					
				}
				
			}
			
			coreUnitBlockSizePosition = coreUnitBlockSizePosition + modTagTotalOffset;
			
			byte[] coreUnitBlockSize = new byte[4];
	    
			coreUnitBlockSize = parsingTools.getfourByteArrayfromFileChannel(fourByteBuffer, inChannel, coreUnitBlockSizePosition);
	    
			int coreUnitBlockSizeInt = parsingTools.getIntfromByteArray(coreUnitBlockSize,0);
	    
			numberOfScenariosinCampaignPosition = coreUnitBlockSizePosition + 4 + coreUnitBlockSizeInt;
	    
			byte[] numberOfScenariosinCampaign = new byte[4];
	    
			numberOfScenariosinCampaign = parsingTools.getfourByteArrayfromFileChannel(fourByteBuffer, inChannel, numberOfScenariosinCampaignPosition);
	    
			int numberOfScenariosinCampaignInt = parsingTools.getIntfromByteArray(numberOfScenariosinCampaign,0);
	    
			campaign.setNumberOfScenarios(numberOfScenariosinCampaignInt);
	    
			/*System.out.println(campaign.getNumberOfScenarios());
	    	System.out.println("-------------------------------------------------");*/
	    
			decisionBlockPosition = numberOfScenariosinCampaignPosition + 4;
	    
			int number_of_Campaign_Ends = 0;
			
			for (int currentScenarioNumber = 0; currentScenarioNumber < campaign.getNumberOfScenarios(); currentScenarioNumber++) {
	    	
				CMx2_CampaignDecisionBlock currentDecisionBlock = new CMx2_CampaignDecisionBlock();
	    	
				byte[] internalBattleNameSize = new byte[2];
	    	
				internalBattleNameSize = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, decisionBlockPosition);
	    	
				int internalBattleNameSizeInt = parsingTools.getShortfromByteArray(internalBattleNameSize,0);
	    	
				if (internalBattleNameSizeInt > 0) {
		    	
					byte[] internalBattleNameStringByteArray = new byte[internalBattleNameSizeInt];
		    	
					internalBattleNamePosition = decisionBlockPosition + 2;
		    	
					winThresholdPosition = internalBattleNamePosition + internalBattleNameSizeInt;
		    	
					internalBattleNameStringByteArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, internalBattleNamePosition, internalBattleNameSizeInt);
		    	
					currentDecisionBlock.setinternalBattleName(parameterAnalyser.convert_ByteArray_to_String(internalBattleNameStringByteArray));
					
					currentDecisionBlock.setInternalBattleNameDefined(true);
		    	
				}else {
		    	
					winThresholdPosition = decisionBlockPosition + 2;
		    	
				}
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, winThresholdPosition);
				currentDecisionBlock.setwinThreshold(parameterAnalyser.analyse_and_return_WinThreshold(singleParameter));
		    
				nextBattleWinPosition = winThresholdPosition + 1;
		    
				byte[] nextBattleWin = new byte[2];
				nextBattleWin = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, nextBattleWinPosition);
				int nextBattlewinInt = parsingTools.getShortfromByteArray(nextBattleWin,0);
				currentDecisionBlock.setnextBattleWin(nextBattlewinInt);
		    
				nextBattleLosePosition = nextBattleWinPosition + 2;
		    
				byte[] nextBattleLose = new byte[2];
				nextBattleLose = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, nextBattleLosePosition);
				int nextBattleLoseInt = parsingTools.getShortfromByteArray(nextBattleLose,0);
				currentDecisionBlock.setnextBattleLose(nextBattleLoseInt);
				
				if (!((nextBattlewinInt != 65535) && (nextBattleLoseInt != 65535))){
					
					number_of_Campaign_Ends++;
					
				}
		    
				minimumRequiredCampaignVictoryPosition = nextBattleLosePosition + 2;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, minimumRequiredCampaignVictoryPosition);
				currentDecisionBlock.setminimumRequiredCampaignVictory(parameterAnalyser.analyse_and_return_VictoryLevel(singleParameter));
		    
				maximumAllowedCampaignVictoryPosition = minimumRequiredCampaignVictoryPosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, maximumAllowedCampaignVictoryPosition);
				currentDecisionBlock.setmaximumAllowedCampaignVictory(parameterAnalyser.analyse_and_return_VictoryLevel(singleParameter));
				
				blueRefitPosition = maximumAllowedCampaignVictoryPosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, blueRefitPosition);
				currentDecisionBlock.setblueRefit(parsingTools.getShortfromByte(singleParameter));
		    
				blueRepairVehiclePosition = blueRefitPosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, blueRepairVehiclePosition);
				currentDecisionBlock.setblueRepairVehicle(parsingTools.getShortfromByte(singleParameter));
		    
				blueResupplyPosition = blueRepairVehiclePosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, blueResupplyPosition);
				currentDecisionBlock.setblueResupply(parsingTools.getShortfromByte(singleParameter));
		    
				blueRestPosition = blueResupplyPosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, blueRestPosition);
				currentDecisionBlock.setblueRest(parsingTools.getShortfromByte(singleParameter));
		    
				redRefitPosition = blueRestPosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, redRefitPosition);
				currentDecisionBlock.setredRefit(parsingTools.getShortfromByte(singleParameter));
		    
				redRepairVehiclePosition = redRefitPosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, redRepairVehiclePosition);
				currentDecisionBlock.setredRepairVehicle(parsingTools.getShortfromByte(singleParameter));
		    
				redResupplyPosition = redRepairVehiclePosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, redResupplyPosition);
				currentDecisionBlock.setredResupply(parsingTools.getShortfromByte(singleParameter));
		    
				redRestPosition = redResupplyPosition + 1;
		    
				singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, redRestPosition);
				currentDecisionBlock.setredRest(parsingTools.getShortfromByte(singleParameter));
		    
				//debugOutputDecisionBlock(currentDecisionBlock);
		    
				scenarioSizePosition = redRestPosition + 1;
		    
				byte[] scenarioSize = new byte[4];
		    
				scenarioSize = parsingTools.getfourByteArrayfromFileChannel(fourByteBuffer, inChannel, scenarioSizePosition);
		    
				int scenarioSizeInt = parsingTools.getIntfromByteArray(scenarioSize,0);
		    
				if (scenarioSizeInt > 0) {
		    	
					byte[] scenarioArray = new byte[scenarioSizeInt];
		    	
					scenarioPosition = scenarioSizePosition + 4;
		    	
					decisionBlockPosition = scenarioSizePosition + 4 + scenarioSizeInt;
		    	
					scenarioArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, scenarioPosition, scenarioSizeInt);
		    	
					if (currentScenarioNumber <= 9) {
		    		
						if(currentDecisionBlock.getInternalBattleNameDefined()){
							
							parsingTools.writebyteArraytoFile(scenarioArray, campaign.getfileName() + " #" + "0" + currentScenarioNumber + "_" + currentDecisionBlock.getinternalBattleName(), campaignScenariosDir, arrayFileType.BTT);
							parsingTools.copyFileToDir("Listing_Layout.xsl", campaignScenariosDir);
							
						}else{
							
							parsingTools.writebyteArraytoFile(scenarioArray, campaign.getfileName() + " #" + "0" + currentScenarioNumber +"_", campaignScenariosDir, arrayFileType.BTT);
							parsingTools.copyFileToDir("Listing_Layout.xsl", campaignScenariosDir);
							
						}
		    		
					}else {
		    		
						if(currentDecisionBlock.getInternalBattleNameDefined()){
							
							parsingTools.writebyteArraytoFile(scenarioArray, campaign.getfileName() + " #" + currentScenarioNumber + "_" + currentDecisionBlock.getinternalBattleName(), campaignScenariosDir, arrayFileType.BTT);
							parsingTools.copyFileToDir("Listing_Layout.xsl", campaignScenariosDir);
							
						}else{
							
							parsingTools.writebyteArraytoFile(scenarioArray, campaign.getfileName() + " #" + currentScenarioNumber + "_", campaignScenariosDir, arrayFileType.BTT);
							parsingTools.copyFileToDir("Listing_Layout.xsl", campaignScenariosDir);
							
						}
		    		
					}

					currentDecisionBlock.setcontainsScenario(true);    	
		    	
				}else {
		    	
					decisionBlockPosition = scenarioSizePosition + 4;
		    	
					currentDecisionBlock.setcontainsScenario(false);
		    	
				}
		    
				campaign.addCampaignDecisionBlock(currentDecisionBlock, currentScenarioNumber);
		    
			}
	    
			campaign.setNumberOfCampaignEnds(number_of_Campaign_Ends);
			
			ScenarioReader scenReader = new ScenarioReader();
		
			if (scenReader.readCampaignScenariosfromSpecifiedDirectory(scenarioDirName)) {
	    
				scenarioList = scenReader.writeOutputtoCampaignDirectory(campaign.getfileName(), campaign.getTitle());
	    	
			}
	    
			Iterator<CMx2_Scenario> scenarioIterator = scenarioList.iterator();
	    
			while (scenarioIterator.hasNext()) {
	    	
				CMx2_Scenario readScenario = ((CMx2_Scenario) scenarioIterator.next());
	    	
				campaign.addCampaignScenario(readScenario, readScenario.getCampaignOrderNumber());
	    	
			}
			
			CampaignTreeWriter treeWriter = new CampaignTreeWriter();
			
			treeWriter.writeCampaignTreeOutput(campaign);
	    
			campaignList.add(campaign);
			
		}
		finally{
			
			if(inFile != null){
				
				inFile.close();
				
			}
			if(inChannel != null){
				
				inChannel.close();
				
			}
			
		}
		
	}
	
	public boolean readCampaignsfromCurrentDirectory() throws IOException {
		
		boolean campaignsRead = false;
		
		File curDir = new File(System.getProperty("user.dir"));
	    
	    for(String currentFile: curDir.list()){
	    	
	    	if (currentFile.contains(".cam")) {
	    		
	    		readSingleCampaign(currentFile);
	    		
	    	    campaignsRead = true;
	    		
	    	}
	    	
	    }
	     
	    return campaignsRead;
	    		
	}
	
	public void writeOutputtoCurrentDirectory() throws IOException {
		
		{
			OutputWriter outputWriter = new OutputWriter();
			
			//outputWriter.writeCampaignHTMLOutput(campaignList);
			
			outputWriter.writeCampaignXMLOutput(campaignList);
			
		}
		
	}
	
	public CampaignReader(){
		
		parsingTools = new ParsingUtilities();
		
	}
	
}
