/* 
    CMx2 ScAnCaDe - Scenario Analyser and Campaign Decompiler
    
    Analysing scenarios and campaigns and creating sortable listings 
    containing meta-information about the analysed scenarios/campaigns.
    Also, extract all scenarios belonging to a campaign from the
    campaign file, enabling stand-alone play of these battles.
    Programmed to support the players and scenario designers of the 
    Combat Mission - CMx2 engine WW2 computer games produced by
    Battlefront Inc. (www.battlefront.com)
    
    Copyright (C) 2014  Michael Wellhfer (michael@wellhoefer.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    Change History:
    
    ---------------------------------------------------------------------
    | VERSION |   DATE     |             DESCRIPTION                    |
    |---------|------------|--------------------------------------------|
    |   1.0   | 10/06/2013 | Initial release version.                   |
    |---------|------------|--------------------------------------------|
    |   1.1   | 15/10/2013 | Added reading out of Region information.   |
    |---------|------------|--------------------------------------------|
    |   1.2   | 17/11/2013 | Added family information / read-out.       |
    |---------|------------|--------------------------------------------|
    |  1.3#1  | 26/11/2013 | Fixed incorrect moding which prevented     |
    |         |            | setting of CampaingBattleNumber when no    |
    |         |            | scenario title has been assigned to a      |
    |         |            | battle in a campaign. This would sub-      |
    |         |            | sequently lead to a crash when creating    |
    |         |            | campaign listing output.                   |
    |---------|------------|--------------------------------------------|
    |  1.3#2  | 15/12/2013 | Added reading out of Year information.     |
    |---------|------------|--------------------------------------------|
    |   1.4   | 09/02/2014 | Added further processing for version /     |
    |         |            | module specific offsets and analysis of    |
    |         |            | required modules.                          |
    |---------|------------|--------------------------------------------|
    |  1.5#1  | 25/02/2014 | Changed processing of campaign btt-file    |
    |         |            | names ("_" as delimiter for reading out of |
    |         |            | battle number).                            |
    |         |            | Also added setting of internalBattleName   |
    |         |            | (name of original btt-file used during     |
    |         |            | creation of campaign) for campaign         |
    |         |            | scenarios if no scenario title has been    |
    |         |            | defined by the designer.                   |
    |---------|------------|--------------------------------------------|
    |  1.5#2  | 05/04/2014 | Added moding necessary for CMRT.           |
    |---------|------------|--------------------------------------------|
    |   1.6   | 27/09/2014 | Added usage of ParsingUtilities provided   |
    |         |            | methods instead of private internal ones.  |
    |---------|------------|--------------------------------------------|
    |   1.7   | 02/11/2014 | Added handling of third module required.   |
    |---------|------------|--------------------------------------------|
    |   1.8   | 22/11/2014 | Added handling of optional modules req'd.  |
    |---------|------------|--------------------------------------------|
    |  1.9#1  | 16/01/2016 | Changed condition for version dependent    |
    |         |            | module information handling.               |
    |---------|------------|--------------------------------------------|
    |  1.9#2  | 27/02/2016 | Added analysis of possible fourth module   |
    |         |            | (for CMBN Battle Pack #1).                 |
    |---------|------------|--------------------------------------------|
*/

package Parser;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.HashSet;
import java.util.Set;

import Data.CMx2_Scenario;
import Output.OutputWriter;

public class ScenarioReader {

	private FileInputStream inFile;
	
	private FileChannel inChannel;
	
	private ByteBuffer oneByteBuffer;
	private ByteBuffer twoByteBuffer;
	private ByteBuffer fourByteBuffer;
	
	private ParameterAnalyser parameterAnalyser;
	
	private final int gameFamilyPosition = 16;
	
	private final int engineVersionPosition = 18;
	
	private final int firstModuleRequired_pre_120_Position = 21;
	
	private final int numberOfModulesRequiredPosition = 25;
	
	private final int firstModuleRequiredPosition = 27;
	
	private final int secondModuleRequiredPosition = 28;
	
	private final int thirdModuleRequiredPosition = 29;
	
	private final int fourthModuleRequiredPosition = 30;
	
	private final int descSizePositionStart = 35;
	
	private int descSizePosition;

	private int environmentPosition;
	private int battleTypePosition;
	private int daylightPosition;
	private int battleSizePosition;
	
	private int titleStringSizePosition;
	
	private int titleStringPosition;
	
	private int scenDescSizePosition;
	private int scenDescPosition;
	
	private int imageSizePosition;
	private int imagePosition;
	
	private int modTagBlockPosition;
	
	private int dataBlockStartPosition;
	
	private int monthPosition;
	private int yearPosition;
	private int dayPosition;
	private int timeOfDayPosition;
	
	private int regionPosition;
	
	private int battleLengthPosition;
	
	private int weatherPosition;
	private int temperaturePosition;
	private int groundConditionPosition;
	
	private int earlyIntelPosition;
	
	private int mapWidthPosition;
	private int mapDepthPosition;
	
	private String campaignDirPrefix = "";
	
	private File scenPicDir;
	
	private Set<CMx2_Scenario> scenarioList = new HashSet<CMx2_Scenario>();
	
	private ParsingUtilities parsingTools;
	
	private void init_File_Channel_Buffers_ParameterAnalyser(String fileName) throws IOException {
		
		File aFile = new File(fileName);
		inFile = new FileInputStream(aFile);
		
	    inChannel = inFile.getChannel();
	    
	    oneByteBuffer = ByteBuffer.allocate(1);
	    
	    twoByteBuffer = ByteBuffer.allocate(2);
	    
	    fourByteBuffer = ByteBuffer.allocate(4);
	    
	    parameterAnalyser = new ParameterAnalyser();
	    
		scenPicDir = new File(campaignDirPrefix + "scenpics");
		
		if (!scenPicDir.exists()) {
			
			scenPicDir.mkdir();
			
		}
		
	}
	
	private void initialiseParameters() {
		
		environmentPosition = descSizePosition + 4;
		battleTypePosition  = descSizePosition + 5;
		daylightPosition    = descSizePosition + 6;
		battleSizePosition  = descSizePosition + 7;
		titleStringSizePosition  = descSizePosition + 8;
		
	}
	
	private void readSingleScenario(String currentScenarioFile) throws IOException {
		
		try{
				
			init_File_Channel_Buffers_ParameterAnalyser(currentScenarioFile);  
	    
			CMx2_Scenario scenario = new CMx2_Scenario();
	    	
			byte singleParameter = 0;
			
			int version = 0;
			int family = 0;
	    	
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, gameFamilyPosition);
			family = parsingTools.getShortfromByte(singleParameter);
			
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, engineVersionPosition);
			version = parsingTools.getShortfromByte(singleParameter);
			
			scenario.setCreatedwithVersion(parameterAnalyser.analyse_and_return_Version(family,version));
	    	
			descSizePosition = descSizePositionStart;
			
			int offset = 0;
			
			int optionalModulesRequiredInt = 0;
			
			if (version > 120) {
				
				offset = offset + 4;
				
				byte[] numberOfModulesRequired = new byte[2];
				byte[] optionalModulesRequired = new byte[2];
				
				numberOfModulesRequired = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, numberOfModulesRequiredPosition);
				
				int numberOfModulesRequiredInt = parsingTools.getShortfromByteArray(numberOfModulesRequired,0);
				
				scenario.setNumberOfModulesRequired(numberOfModulesRequiredInt);
				
				switch(numberOfModulesRequiredInt){
				
					case 0:
									
						optionalModulesRequired = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, numberOfModulesRequiredPosition + 2);
						
						optionalModulesRequiredInt = parsingTools.getShortfromByteArray(optionalModulesRequired,0);
						
						if(optionalModulesRequiredInt == 1){
							
							int firstOptionalModuleRequired = 0;
							int secondOptionalModuleRequired = 0;
							
							firstOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, numberOfModulesRequiredPosition + 4));
							secondOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, numberOfModulesRequiredPosition + 5));
							
							scenario.setOptionalModulesRequired(parameterAnalyser.analyse_and_return_Module_Required(firstOptionalModuleRequired, family) + " or " + parameterAnalyser.analyse_and_return_Module_Required(secondOptionalModuleRequired, family));
							
							offset = offset + 2;
						}
						
						break;
				
				    case 1:
				    	
				    	int firstModuleRequired = 0;
				    	firstModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	scenario.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequired, family));
				    	
				    	offset = offset + 1;
				    	
						optionalModulesRequired = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, firstModuleRequiredPosition + 1);
						
						optionalModulesRequiredInt = parsingTools.getShortfromByteArray(optionalModulesRequired,0);
						
						if(optionalModulesRequiredInt == 1){
							
							int firstOptionalModuleRequired = 0;
							int secondOptionalModuleRequired = 0;
							
							firstOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition + 3));
							secondOptionalModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition + 4));
							
							scenario.setOptionalModulesRequired(parameterAnalyser.analyse_and_return_Module_Required(firstOptionalModuleRequired, family) + " or " + parameterAnalyser.analyse_and_return_Module_Required(secondOptionalModuleRequired, family));
							
							offset = offset + 2;
						}
				    	
				    	break;
				    
				    case 2:
				    	
				    	int firstModuleRequiredOfTwo = 0;
				    	firstModuleRequiredOfTwo = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	scenario.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequiredOfTwo, family));
				    	
				    	int secondModuleRequired = 0;
				    	secondModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, secondModuleRequiredPosition));
				    	scenario.setSecondModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(secondModuleRequired, family));
				    	
				    	offset = offset + 2;
				    	
				    	break;
				    	
				    case 3:
				    	
				    	int firstModuleRequiredOfThree = 0;
				    	firstModuleRequiredOfThree = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	scenario.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequiredOfThree, family));
				    	
				    	int secondModuleRequiredOfThree = 0;
				    	secondModuleRequiredOfThree = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, secondModuleRequiredPosition));
				    	scenario.setSecondModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(secondModuleRequiredOfThree, family));
				    	
				    	int thirdModuleRequired = 0;
				    	thirdModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, thirdModuleRequiredPosition));
				    	scenario.setThirdModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(thirdModuleRequired, family));
				    	
				    	offset = offset + 3;
				    	
				    	break;
				    	
				    case 4:
				    	
				    	int firstModuleRequiredOfFour = 0;
				    	firstModuleRequiredOfFour = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequiredPosition));
				    	scenario.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequiredOfFour, family));
				    	
				    	int secondModuleRequiredOfFour = 0;
				    	secondModuleRequiredOfFour = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, secondModuleRequiredPosition));
				    	scenario.setSecondModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(secondModuleRequiredOfFour, family));
				    	
				    	int thirdModuleRequiredOfFour = 0;
				    	thirdModuleRequiredOfFour = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, thirdModuleRequiredPosition));
				    	scenario.setThirdModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(thirdModuleRequiredOfFour, family));
				    	
				    	int fourthModuleRequired = 0;
				    	fourthModuleRequired = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, fourthModuleRequiredPosition));
				    	scenario.setFourthModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(fourthModuleRequired, family));
				    	
				    	offset = offset + 4;
				    	
				    	break;
				    	
				    default: break;
				
				}
				
			} else {
				
				int firstModuleRequired_pre_120 = 0;
				
				firstModuleRequired_pre_120 = parsingTools.getShortfromByte(parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, firstModuleRequired_pre_120_Position));
				
				if (firstModuleRequired_pre_120 == 1){
					
					scenario.setNumberOfModulesRequired(1);
					scenario.setFirstModuleRequired(parameterAnalyser.analyse_and_return_Module_Required(firstModuleRequired_pre_120, family));					
					
				}
				
			}
			
			descSizePosition = descSizePosition + offset;
			
			initialiseParameters();
  
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, environmentPosition);
			scenario.setEnvironment(parameterAnalyser.analyse_and_return_Environment(singleParameter));
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, battleTypePosition);
			scenario.setBattleType(parameterAnalyser.analyse_and_return_Battle_Type(singleParameter, family));
	    	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, daylightPosition);
			scenario.setDaylight(parameterAnalyser.analyse_and_return_Daylight(singleParameter));
	    	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, battleSizePosition);
			scenario.setBattleSize(parameterAnalyser.analyse_and_return_Battle_Size(singleParameter));
	    
			byte[] titleStringSize = new byte[2];

			titleStringSize = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, titleStringSizePosition);
	    
			int titleStringSizeInt = parsingTools.getShortfromByteArray(titleStringSize,0);
	    
			boolean titleStringDefined = false;
			
			if (titleStringSizeInt > 0) {
	    	
				byte[] titleStringByteArray = new byte[titleStringSizeInt];
	    	
				titleStringPosition = titleStringSizePosition + 2;
	    	
				scenDescSizePosition = titleStringPosition + titleStringSizeInt;
	    	
				titleStringByteArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, titleStringPosition, titleStringSizeInt);
    	
				scenario.setTitle(parameterAnalyser.convert_ByteArray_to_String(titleStringByteArray));
				
				titleStringDefined = true;

	    	
			} else {
	    	
				scenDescSizePosition = titleStringSizePosition + 2;
	    	
			}
			
			if (currentScenarioFile.contains("#")){
	    		
				String battleNumber = currentScenarioFile.substring(currentScenarioFile.indexOf("#"), currentScenarioFile.indexOf("_"));
    		
				scenario.setCampaignBattleNumber("[Campaign Battle " + battleNumber + "]");
    		
				scenario.setCampaignOrderNumber(Integer.valueOf((battleNumber.substring(1))));
				
				if(!titleStringDefined){
					
					scenario.setTitle(currentScenarioFile.substring(currentScenarioFile.indexOf("_") + 1, currentScenarioFile.indexOf(".")));
					
				}
    		
			}
	    
			if (currentScenarioFile.contains("/")) {
	    	
				int startPos = currentScenarioFile.lastIndexOf("/") + 1;
				int endPos = currentScenarioFile.lastIndexOf(".btt");
	    	
				scenario.setfileName(currentScenarioFile.substring(startPos, endPos));
	    	
			} else {
	    	
				int startPos = 0;
				int endPos = currentScenarioFile.lastIndexOf(".btt");
	    	
				scenario.setfileName(currentScenarioFile.substring(startPos, endPos));
	    	
			}
	    
			if (scenario.getfileName().contains("#")){
			
				scenario.setbitmapFileName((scenario.getfileName().substring(0,scenario.getfileName().lastIndexOf("#") - 1)) + scenario.getCampaignOrderNumber());
				//scenario.setbitmapFileName((scenario.getfileName().substring(0,scenario.getfileName().lastIndexOf("#") - 1)) + (scenario.getfileName().substring(scenario.getfileName().lastIndexOf("#") + 1)));
			
			} else {
			
				scenario.setbitmapFileName(scenario.getfileName());
			
			}
	    
			byte[] scenDescStringSize = new byte[2];
	    
			scenDescStringSize = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, scenDescSizePosition);
	    
			int scenDescStringSizeInt = parsingTools.getShortfromByteArray(scenDescStringSize,0);
	    
			if (scenDescStringSizeInt > 0) {
	    	
				byte[] scenDescStringByteArray = new byte[scenDescStringSizeInt];
	    	
				scenDescPosition = scenDescSizePosition + 2;
	    	
				imageSizePosition = scenDescPosition + scenDescStringSizeInt;
	    	
				scenDescStringByteArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, scenDescPosition, scenDescStringSizeInt);
	    	
				scenario.setDescription(parameterAnalyser.convert_ByteArray_to_String(scenDescStringByteArray));
	    	
			} else {
	    	
				imageSizePosition = scenDescSizePosition + 2;
	    	
			}
	    
			byte[] imageSize = new byte[4];
	    
			imageSize = parsingTools.getfourByteArrayfromFileChannel(fourByteBuffer, inChannel, imageSizePosition);
	    
			int imageSizeInt = parsingTools.getIntfromByteArray(imageSize,0);
	    
			if (imageSizeInt > 0) {
	    	
				//byte[] bitmapImageArray = new byte[imageSizeInt];
	    	
				imagePosition = imageSizePosition + 4;
	    	
				dataBlockStartPosition = imagePosition + imageSizeInt;
	    	
				//bitmapImageArray = getflexibleByteArrayfromFileChannel(imagePosition, imageSizeInt);
	    	
				parsingTools.writebyteArraytoFile(parsingTools.getflexibleByteArrayfromFileChannel(inChannel, imagePosition, imageSizeInt), scenario.getbitmapFileName(), scenPicDir, arrayFileType.BMP);
				
				//bitmapImageArray = null;
	    	
			} else {
	    	
				dataBlockStartPosition = imageSizePosition + 4;
	    	
			}
	    
			// mod tag specific 
			int modTagTotalOffset = 0;
			
			if (version >= 130){
				
				modTagBlockPosition = dataBlockStartPosition + 3;
				
				modTagTotalOffset = 5;
				
				byte[] numberOfModTags = new byte[4];
				
				numberOfModTags = parsingTools.getfourByteArrayfromFileChannel(fourByteBuffer, inChannel, modTagBlockPosition);
				
				int numberOfModTagsInt = parsingTools.getIntfromByteArray(numberOfModTags,0);
				
				if (numberOfModTagsInt > 0){
					
					String[] modTagsArray = new String[numberOfModTagsInt];
					
					int currentModTagSizePosition = modTagBlockPosition + 4;
					
					for(int currentModTagIndex = 0; currentModTagIndex < numberOfModTagsInt; currentModTagIndex++){
						
						byte[] currentModTagSize = new byte[2];
						
						currentModTagSize = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, currentModTagSizePosition);
						
						int currentModTagSizeInt = parsingTools.getShortfromByteArray(currentModTagSize,0);
						
						if (currentModTagSizeInt>0){
							
							byte[] currentModTagByteArray = new byte[currentModTagSizeInt];
							
							currentModTagByteArray = parsingTools.getflexibleByteArrayfromFileChannel(inChannel, currentModTagSizePosition + 2, currentModTagSizeInt);
							
							modTagsArray[currentModTagIndex] = parameterAnalyser.convert_ByteArray_to_String(currentModTagByteArray);
							
							currentModTagSizePosition = currentModTagSizePosition + 2 + currentModTagSizeInt;
							
							modTagTotalOffset = modTagTotalOffset + 2 + currentModTagSizeInt;
							
						} else {
							
							currentModTagSizePosition = currentModTagSizePosition + 2;
							
							modTagTotalOffset = modTagTotalOffset + 2;
							
						}
						
					}
			
					scenario.setModTags(modTagsArray);
					
				}
				
			}
			
			monthPosition = dataBlockStartPosition + 2 + modTagTotalOffset;
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, monthPosition);
			scenario.setMonth(parameterAnalyser.analyse_and_return_Month(singleParameter));
			
			yearPosition = monthPosition + 1;
			
			byte[] yearByteArray = new byte[2];
		    
			yearByteArray = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, yearPosition);
			int year = parsingTools.getShortfromByteArray(yearByteArray,0);
			scenario.setYear(parameterAnalyser.analyse_and_return_Year(year));
			
			regionPosition = yearPosition + 2;
			
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, regionPosition);
			scenario.setRegion(parameterAnalyser.analyse_and_return_Region(singleParameter, family, version));
	    
			dayPosition = regionPosition + 1;
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, dayPosition);
			scenario.setDay(parameterAnalyser.analyse_and_return_Day(singleParameter));
	    
			timeOfDayPosition = dayPosition + 1;
	    
			byte[] timeOfDayByteArray = new byte[2];
	    
			timeOfDayByteArray = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, timeOfDayPosition);
			int timeofDayinMinutes = parsingTools.getShortfromByteArray(timeOfDayByteArray,0);
			scenario.setTime(parameterAnalyser.analyse_and_return_TimeofDay_or_BattleLength(timeofDayinMinutes));
	    
			battleLengthPosition = timeOfDayPosition + 2;
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, battleLengthPosition);
			int battleLengthinMinutes = parsingTools.getShortfromByte(singleParameter);
			scenario.setBattleLength(parameterAnalyser.analyse_and_return_TimeofDay_or_BattleLength(battleLengthinMinutes));
	    
			weatherPosition = battleLengthPosition + 2;
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, weatherPosition);
			scenario.setWeather(parameterAnalyser.analyse_and_return_Weather(singleParameter));
	    
			temperaturePosition = weatherPosition + 1;
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, temperaturePosition);
			scenario.setTemperature(parameterAnalyser.analyse_and_return_Temperature(singleParameter));
	    
			groundConditionPosition = temperaturePosition + 1;
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, groundConditionPosition);
			scenario.setGroundCondition(parameterAnalyser.analyse_and_return_GroundCondition(singleParameter));
	    
			earlyIntelPosition = groundConditionPosition + 1;
	    
			singleParameter = parsingTools.getSingleParameterfromFileChannel(oneByteBuffer, inChannel, earlyIntelPosition);
			scenario.setEarlyIntel(parameterAnalyser.analyse_and_return_EarlyIntel(singleParameter));
	    
			mapWidthPosition = earlyIntelPosition + 1;
	    
			byte[] mapWidthByteArray = new byte[2];
	    
			mapWidthByteArray = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, mapWidthPosition);
			int mapWidthinMeters = parsingTools.getShortfromByteArray(mapWidthByteArray,0);
			scenario.setMapWidth(String.valueOf(mapWidthinMeters));
	    
			mapDepthPosition = mapWidthPosition + 2;
	    
			byte[] mapDepthByteArray = new byte[2];
	    
			mapDepthByteArray = parsingTools.gettwoByteArrayfromFileChannel(twoByteBuffer, inChannel, mapDepthPosition);
			int mapDepthinMeters = parsingTools.getShortfromByteArray(mapDepthByteArray,0);
			scenario.setMapDepth(String.valueOf(mapDepthinMeters));
	    
			scenario.setMapArea();
	    
			scenario.setSizeModifier(parameterAnalyser.analyse_and_return_Size_Modifier(scenario.getBattleLength(), scenario.getBattleSize()));
	    
			scenarioList.add(scenario);
		
		}finally{
			
			if(inFile != null){
				
				inFile.close();
				
			}
			if(inChannel != null){
				
				inChannel.close();
				
			}
			
		}
		
	}
	
	public boolean readScenariosfromCurrentDirectory() throws IOException {
		
		boolean scenariosRead = false;
		
		File curDir = new File(System.getProperty("user.dir"));
	    
	    for(String currentFile: curDir.list()){
	    	
	    	if (currentFile.contains(".btt")) {
	    		
	    		readSingleScenario(currentFile);
	    		
	    	    scenariosRead = true;
	    		
	    	}
	    	
	    }
	    
	    return scenariosRead;
	    		
	}
	
	public void writeOutputtoCurrentDirectory() throws IOException {
		
		{
			
			OutputWriter outputWriter = new OutputWriter();
			
			//outputWriter.writeScenarioHTMLOutput(scenarioList, false, campaignDirPrefix, "", "");
			
			outputWriter.writeScenarioXMLOutput(scenarioList, false, campaignDirPrefix, "", "");
			
		}
		
	}
	
	public boolean readCampaignScenariosfromSpecifiedDirectory(String subdir) throws IOException {
		
		boolean scenariosRead = false;
		
		campaignDirPrefix = subdir + "/";
		
		File curDir = new File(campaignDirPrefix);
	    
	    for(String currentFile: curDir.list()){
	    	
	    	if (currentFile.contains(".btt")) {
	    		
	    		readSingleScenario(campaignDirPrefix + currentFile);
	    		
	    	    scenariosRead = true;
	    		
	    	}
	    	
	    }
	    
	    return scenariosRead;
		
	}
	
	public Set<CMx2_Scenario> writeOutputtoCampaignDirectory(String campaignFileName, String campaignTitle) throws IOException {
		
		{
			
			OutputWriter outputWriter = new OutputWriter();
			
			//outputWriter.writeScenarioHTMLOutput(scenarioList, true, campaignDirPrefix, campaignFileName, campaignTitle);
			
			outputWriter.writeScenarioXMLOutput(scenarioList, true, campaignDirPrefix, campaignFileName, campaignTitle);
			
			return scenarioList;
			
		}
		
	}
	
	public ScenarioReader(){
		
		parsingTools = new ParsingUtilities();
		
	}
	
}
